@@ -79,7 +79,7 @@ split_f <- function(data, validation, seed){
 
 
 # functtion to define the objective function
-objfn <- function(initp, Q, y, family, covrts){
+objfn <- function(initp, Q, y, family, covrts, b1_constr){
 
   b0 = initp[1]
   b1 = initp[2]
@@ -100,7 +100,7 @@ objfn <- function(initp, Q, y, family, covrts){
 
 
 # function to define the equality constraint
-linconst = function(initp, Q, y, family, covrts){
+linconst = function(initp, Q, y, family, covrts, b1_constr){
 
   wsum=sum(initp[3:(dim(Q)[2] + 2)])
 
@@ -109,27 +109,36 @@ linconst = function(initp, Q, y, family, covrts){
 
 
 # function to determine bounded prameters
-bounded_param = function(initp, Q, y, family, covrts){
+bounded_param = function(initp, Q, y, family, covrts, b1_constr){
 
-  b_p = initp[3:(dim(Q)[2] + 2)]
+  if (b1_constr) b_p = initp[2:(dim(Q)[2] + 2)]
+  else b_p = initp[3:(dim(Q)[2] + 2)]
 
   return(b_p)
 }
 
 
 # function to define the lower bounds
-LBound = function(Q, b1_pos){
+LBound = function(Q, b1_pos, b1_constr){
 
-  LB = rep(0, dim(Q)[2])
+  if (b1_constr) {
+    b1l = ifelse(b1_pos, 0, -Inf)
+    LB = c(b1l, rep(0, dim(Q)[2]))
+  }
+  else LB = rep(0, dim(Q)[2])
 
   return(LB)
 }
 
 
 # function to define the upper bounds
-UBound = function(Q, b1_pos){
+UBound = function(Q, b1_pos, b1_constr){
 
-  UB = rep(1, dim(Q)[2])
+  if (b1_constr) {
+    b1u = ifelse(b1_pos, Inf, 0)
+    UB = c(b1u, rep(1, dim(Q)[2]))
+  }
+  else UB = rep(1, dim(Q)[2])
 
   return(UB)
 }
@@ -146,7 +155,7 @@ values.0 = function(lenp, y, covrts, b1_pos, family){
     names(val.0) = c("b0", "b1", paste0("w", 1:lenp))
   }
   else {
-    if (family == "gaussian") fit = glm(y ~ covrts, family = gaussian(link = "identity"))
+    if (family == "gaussian") fit = lm(y ~ covrts)
     else if (family == "binomial") fit = glm(y ~ covrts, family = binomial(link = "logit"))
     bj = coef(fit)[-1]
     val.0 = c(0, b1_0, rep(1/lenp, lenp), bj)
@@ -158,23 +167,24 @@ values.0 = function(lenp, y, covrts, b1_pos, family){
 
 
 # optimization function to estimate the weights
-optim.f <- function(Q, y, b1_pos, family, covrts){
+optim.f <- function(Q, y, b1_pos, b1_constr, family, covrts){
   Q = as.matrix(Q)
   if(dim(covrts)[2] > 0) covrts = as.matrix(covrts)
   lenp = dim(Q)[2]
   initp = values.0(lenp, y, covrts, b1_pos, family)
-  LowB = LBound(Q, b1_pos)
-  UpB = UBound(Q, b1_pos)
+  LowB = LBound(Q, b1_pos, b1_constr)
+  UpB = UBound(Q, b1_pos, b1_constr)
 
-  opt_res = suppressWarnings(solnp(pars = initp, fun = objfn, eqfun = linconst, eqB = 1,
+  opt_res = tryCatch(solnp(pars = initp, fun = objfn, eqfun = linconst, eqB = 1,
                                    ineqfun = bounded_param, ineqLB = LowB, ineqUB = UpB,
                                    control = list(trace = 0), Q = Q, y = y, family = family,
-                                   covrts = covrts))
-
+                                   covrts = covrts, b1_constr = b1_constr),
+					error = function(e) NULL)
+  if(!is.null(opt_res)) {
   par_opt = opt_res$pars
   conv = opt_res$convergence
   nfuneval = opt_res$nfuneval
-
+  } else {par_opt=initp; conv=2; nfuneval=0}
   out = list(par_opt, conv, nfuneval)
   names(out) = c("par_opt", "conv", "nfuneval")
 
@@ -198,7 +208,7 @@ model.fit <- function(Q, y, wghts, family, covrts, wqs2){
     names(new_data)[c((dim(wqs)[2] + 2):dim(new_data)[2])] = names(covrts)
   }
 
-  if (family == "gaussian") m_f = glm(y ~ ., data = new_data, family = gaussian(link = "identity"))
+  if (family == "gaussian") m_f = lm(y ~ ., data = new_data)
   else if (family == "binomial") m_f = glm(y ~ ., data = new_data, family = binomial(link = "logit"))
 
   mf_out = list(wqs, m_f)
@@ -211,12 +221,12 @@ model.fit <- function(Q, y, wghts, family, covrts, wqs2){
     wqs = cbind(wqs, wqs_2)
     new_data = cbind(new_data, wqs_2)
 
-    if (family == "gaussian") m_f2 = glm(y ~ ., data = new_data, family = gaussian(link = "identity"))
+    if (family == "gaussian") m_f2 = lm(y ~ ., data = new_data)
     else if (family == "binomial") m_f2 = glm(y ~ ., data = new_data, family = binomial(link = "logit"))
 
     aov = anova(m_f, m_f2, test = "Chisq")
 
-    mf_out = list(wqs, m_f, m_f2, aov)
+    mf_out = list(wqs[, 1], m_f, m_f2, aov)
     names(mf_out) = c("wqs", "m_f", "m_f2", "aov")
   }
 
@@ -225,7 +235,7 @@ model.fit <- function(Q, y, wghts, family, covrts, wqs2){
 
 
 # function that calls the optimization function and the function to fit the model for each bootstrap sample
-par.modl.est <- function(data, y_name, q_name, cov_name, b, b1_pos, family, seed){
+par.modl.est <- function(data, y_name, q_name, cov_name, b, b1_pos, b1_constr, family, seed){
 
   index_b = vector("list", b)
   param = vector("list", b)
@@ -243,11 +253,11 @@ par.modl.est <- function(data, y_name, q_name, cov_name, b, b1_pos, family, seed
 
     # create bootstrap sample
     index_b[[i]] = sample(1:nrow(data), nrow(data), replace=TRUE)
-    data_b = data[index_b[[i]],]
+    data_b <- data[index_b[[i]],]
 
     # calculate parameters
-    param[[i]] = optim.f(data_b[, q_name, drop = FALSE], data_b[, y_name, drop = FALSE], b1_pos,
-                         family, data_b[, cov_name, drop = FALSE])
+    param[[i]] = optim.f(data_b[, q_name, drop = FALSE], data_b[, y_name, drop = FALSE],
+                         b1_pos, b1_constr, family, data_b[, cov_name, drop = FALSE])
 
     # fit the wqs model for each bootstrap sample
     b_fit = model.fit(data_b[, q_name, drop = FALSE], data_b[, y_name, drop = FALSE],
@@ -261,6 +271,9 @@ par.modl.est <- function(data, y_name, q_name, cov_name, b, b1_pos, family, seed
     nfuneval[i] = param[[i]]$nfuneval
   }
 
+  n_non_conv = sum(conv == 2)
+  print(paste0("The optimization function did not converge ", n_non_conv, " times"))
+
   par_model_out = list(wght_matrix, b1, conv, p_val, index_b, nfuneval)
   names(par_model_out) = c("wght_matrix", "b1", "conv", "p_val", "index_b", "nfuneval")
 
